#!/usr/bin/python3
from datetime import datetime
from sys import stdout, stdin
from time import sleep
import os


def cls():
	"""Clear screen (unix)."""
	os.system('clear')


class FG:
	"""Foreground class."""

	black = "\u001b[30m"
	red = "\u001b[31m"
	green = "\u001b[32m"
	yellow = "\u001b[33m"
	blue = "\u001b[34m"
	magenta = "\u001b[35m"
	cyan = "\u001b[36m"
	white = "\u001b[37m"

	@staticmethod
	def rgb(r: int, g: int, b: int) -> str:
		"""Function for convert rgb to ansi color code.
		
		Arguments:
		---------
		 r: int - red color (0-255)
		 g: int - green color (0-255)
		 b: int - blue color (0-255)

		Return:
		------
		+ str - ansi unicode color code

		"""
		return f"\u001b[38;2;{r};{g};{b}m"

class BG:
	"""Background class."""

	black = "\u001b[40m"
	red = "\u001b[41m"
	green = "\u001b[42m"
	yellow = "\u001b[43m"
	blue = "\u001b[44m"
	magenta = "\u001b[45m"
	cyan = "\u001b[46m"
	white = "\u001b[47m"

	@staticmethod
	def rgb(r: int, g: int, b: int) -> str:
		"""Function for convert rgb to ansi color code.
		
		Arguments:
		---------
		 r: int - red color (0-255)
		 g: int - green color (0-255)
		 b: int - blue color (0-255)

		Return:
		------
		+ str - ansi unicode color code

		"""
		return f"\u001b[48;2;{r};{g};{b}m"


class Style:
	"""Style class."""

	reset = "\u001b[0m"
	bold = "\u001b[1m"
	dim = "\u001b[2m"
	italic = "\u001b[3m"
	underline = "\u001b[4m"
	reverse = "\u001b[7m"
	clear = "\u001b[2J"
	clearline = "\u001b[2K"
	up = "\u001b[1A"
	down = "\u001b[1B"
	right = "\u001b[1C"
	left = "\u001b[1D"
	nextline = "\u001b[1E"
	prevline = "\u001b[1F"
	top = "\u001b[0;0H"
	
	@staticmethod
	def to(x, y):
		"""Move cursor to x, y."""
		return f"\u001b[{y};{x}H"
	
	@staticmethod
	def write(text="\n"):
		"""Print to stdout."""
		stdout.write(text)
		stdout.flush()
	
	@staticmethod
	def writew(text="\n", wait=0.01):
		"""Print (typewrite effect)."""
		for char in text:
			stdout.write(char)
			stdout.flush()
			sleep(wait)
  	
	@staticmethod
	def read(begin=""):
		"""Read input from keyboard."""
		text = ""
		stdout.write(begin)
		stdout.flush()
		while True:
			char = ord(stdin.read(1))
      
		if char == 3: return
		elif char in (10, 13): return text
		else: text += chr(char)
	
	@staticmethod
	def readw(begin="", wait=0.5):
		"""Read input with wait."""
		text = ""

		for char in begin:
			stdout.write(char)
			stdout.flush()
			sleep(wait)
		
		while True:
			char = ord(stdin.read(1))
      
			if char == 3: return
			elif char in (10, 13): return text
			else: text += chr(char)


def info_message(text: str, highlight: bool=False) -> str:
	"""Print info message.

	Arguments:
	---------
	 + text: str - text of message
	+ highlight: bool=False - need to highlight background

	"""
	prefix = f'{BG.green}{FG.black}' if highlight else f'{FG.green}'
	print(f'{prefix}[DEBUG {datetime.now()}]{Style.reset} {text}{Style.reset}')


def warn_message(text: str, highlight: bool=False) -> str:
	"""Print warn message.

	Arguments:
	---------
	 + text: str - text of message
	+ highlight: bool=False - need to highlight background

	"""
	prefix = f'{BG.yellow}{FG.black}' if highlight else f'{FG.yellow}'
	print(f'{prefix}[DEBUG {datetime.now()}]{Style.reset} {text}{Style.reset}')


def error_message(text: str, highlight: bool=False) -> str:
	"""Print error message.

	Arguments:
	---------
	 + text: str - text of message
	+ highlight: bool=False - need to highlight background

	"""
	prefix = f'{BG.red}{FG.black}' if highlight else f'{FG.red}'
	print(f'{prefix}[DEBUG {datetime.now()}]{Style.reset} {text}{Style.reset}')


def debug_message(text: str, highlight: bool=False) -> str:
	"""Print debug message.

	Arguments:
	---------
	 + text: str - text of message
	+ highlight: bool=False - need to highlight background

	"""
	prefix = f'{BG.magenta}{FG.black}' if highlight else f'{FG.magenta}'
	print(f'{prefix}[DEBUG {datetime.now()}]{Style.reset} {text}{Style.reset}')


def other_message(text: str, msg_type: str, highlight: bool=False) -> str:
	"""Print messages.

	Arguments:
	---------
	 + text: str - text of message
	 + msg_type: str - type of message (ex. other)
	+ highlight: bool=False - need to highlight background

	"""
	prefix = f'{BG.cyan}{FG.black}' if highlight else f'{FG.cyan}'
	print(f'{prefix}[{msg_type.upper()} {datetime.now()}]{Style.reset} {text}{Style.reset}')


def run_exception(text: str, highlight: bool=False):
	"""Run exception message.

	Arguments:
	---------
	 + text: str - text of message
	+ highlight: bool=False - need to highlight background

	"""
	prefix = f'{BG.red}{FG.black}' if highlight else f'{FG.red}'
	print(f'{Style.bold}{prefix}[EXCEPTION {datetime.now()}]{Style.reset} {text}{Style.reset}')
	raise Exception(text)



#!venv/bin/python3
"""Paintlog Example File.

Copyright Alexeev Bronislav (C) 2024
BSD 3 Clause License
"""
from time import monotonic
import ast
import logging
import inspect
import pprint
import sys
import warnings
from datetime import datetime
import functools
from contextlib import contextmanager
from os.path import basename, realpath
from textwrap import dedent
import colorama
import executing
from pygments import highlight
from pygments.formatters import Terminal256Formatter
from pygments.lexers import PythonLexer as PyLexer, Python3Lexer as Py3Lexer

from paintlog.paint import debug_message, info_message, warn_message, error_message, other_message, run_exception
from paintlog.color_highlight import CatppuccinMocha


PYTHON2 = (sys.version_info[0] == 2)

_absent = object()
DEFAULT_THEME = CatppuccinMocha


@contextmanager
def supportTerminalColorsInWindows():
	"""Support terminal colors in Windows OS with colorama."""
	colorama.init()
	yield
	colorama.deinit()


def stderrPrint(*args):
	"""Print to stderr."""
	print(*args)


def isLiteral(s):
	"""Check string if literal."""
	try:
		ast.literal_eval(s)
	except Exception:
		return False
	return True


def bindStaticVariable(name, value):
    def decorator(fn):
        setattr(fn, name, value)
        return fn
    return decorator


@bindStaticVariable('formatter', Terminal256Formatter(style=DEFAULT_THEME))
@bindStaticVariable(
	'lexer', PyLexer(ensurenl=False) if PYTHON2 else Py3Lexer(ensurenl=False))
def colorize(s):
	"""Colorize with pygments."""
	self = colorize
	return highlight(s, self.lexer, self.formatter)


def colorized_stderr_print(obj):
	"""Colorized stderr print."""
	for s in obj.split('; '):
		if not s.startswith('pydbg_obj |'):
			s = f'pydbg_obj | {s}'
		colored = colorize(s)

		with supportTerminalColorsInWindows():
			stderrPrint(colored)


DEFAULT_PREFIX = 'pydbg_obj | '
DEFAULT_LINE_WRAP_WIDTH = 80  # Characters.
DEFAULT_CONTEXT_DELIMITER = '~ '
DEFAULT_OUTPUT_FUNCTION = colorized_stderr_print
DEFAULT_ARG_TO_STRING_FUNCTION = pprint.pformat


NO_SOURCE_AVAILABLE_WARNING_MESSAGE = (
	'Failed to access the underlying source code for analysis. Was ic() '
	'invoked in a REPL (e.g. from the command line), a frozen application '
	'(e.g. packaged with PyInstaller), or did the underlying source code '
	'change during execution?')


def callOrValue(obj):
	"""Call or value."""
	return obj() if callable(obj) else obj


class Source(executing.Source):
	"""Source."""

	def get_text_with_indentation(self, node):
		"""Get text with indents."""
		result = self.asttokens().get_text(node)
		if '\n' in result:
			result = ' ' * node.first_token.start[1] + result
			result = dedent(result)
		result = result.strip()
		return result


def prefixLines(prefix, s, startAtLine=0):
	"""Prefix lines."""
	lines = s.splitlines()

	for i in range(startAtLine, len(lines)):
		lines[i] = prefix + lines[i]

	return lines


def prefixFirstLineIndentRemaining(prefix, s):
	"""First line indent remaining prefix."""
	indent = ' ' * len(prefix)
	lines = prefixLines(indent, s, startAtLine=1)
	lines[0] = prefix + lines[0]
	return lines


def formatPair(prefix, arg, value):
	"""Formatting pair."""
	if arg is _absent:
		argLines = []
		valuePrefix = prefix
	else:
		argLines = prefixFirstLineIndentRemaining(prefix, arg)
		valuePrefix = argLines[-1] + ': '

	looksLikeAString = (value[0] + value[-1]) in ["''", '""']
	if looksLikeAString:  # Align the start of multiline strings.
		valueLines = prefixLines(' ', value, startAtLine=1)
		value = '\n'.join(valueLines)

	valueLines = prefixFirstLineIndentRemaining(valuePrefix, value)
	lines = argLines[:-1] + valueLines
	return '\n'.join(lines)


def singledispatch(func):
	"""Single dispatch function."""
	if "singledispatch" not in dir(functools):
		def unsupport_py2(*args, **kwargs):
			raise NotImplementedError(
				"functools.singledispatch is missing in " + sys.version
			)
		func.register = func.unregister = unsupport_py2
		return func

	func = functools.singledispatch(func)

	# add unregister based on https://stackoverflow.com/a/25951784
	closure = dict(zip(func.register.__code__.co_freevars, 
					   func.register.__closure__))
	registry = closure['registry'].cell_contents
	dispatch_cache = closure['dispatch_cache'].cell_contents
	def unregister(cls):
		del registry[cls]
		dispatch_cache.clear()
	func.unregister = unregister
	return func


@singledispatch
def argumentToString(obj):
	"""Convert argument to string."""
	s = DEFAULT_ARG_TO_STRING_FUNCTION(obj)
	s = s.replace('\\n', '\n')  # Preserve string newlines in output.
	return s


class PyDBG_Obj:
	"""Advanced print for debuging.
	
	>>> pydbg_obj | num: 12
	            	float_int: 12.12
	            	string: 'Hello'
	            	boolean: True
	            	list_array: [1, 2, 3, 'Hi', True, 12.2]
	            	dictionary: {1: 'HELLO', 2: 'WORLD'}
	
	"""

	_pairDelimiter = '; '
	lineWrapWidth = DEFAULT_LINE_WRAP_WIDTH
	contextDelimiter = DEFAULT_CONTEXT_DELIMITER

	def __init__(self, prefix=DEFAULT_PREFIX,
				 outputFunction=DEFAULT_OUTPUT_FUNCTION,
				 argToStringFunction=argumentToString, includeContext=False,
				 contextAbsPath=False):
		"""Initialization."""
		self.enabled = True
		self.prefix = prefix
		self.includeContext = includeContext
		self.outputFunction = outputFunction
		self.argToStringFunction = argToStringFunction
		self.contextAbsPath = contextAbsPath

	def __call__(self, *args):
		"""Call magic method."""
		if self.enabled:
			callFrame = inspect.currentframe().f_back
			self.outputFunction(self._format(callFrame, *args))

		if not args:
			passthrough = None
		elif len(args) == 1:
			passthrough = args[0]
		else:
			passthrough = args

		return passthrough

	def format(self, *args):
		"""Format arguments."""
		callFrame = inspect.currentframe().f_back
		out = self._format(callFrame, *args)
		return out

	def _format(self, callFrame, *args):
		"""Format helper function."""
		prefix = callOrValue(self.prefix)

		context = self._formatContext(callFrame)
		if not args:
			time = self._formatTime()
			out = prefix + context + time
		else:
			if not self.includeContext:
				context = ''
			out = self._formatArgs(
				callFrame, prefix, context, args)

		return out

	def _formatArgs(self, callFrame, prefix, context, args):
		"""Format arguments."""
		callNode = Source.executing(callFrame).node
		if callNode is not None:
			source = Source.for_frame(callFrame)
			sanitizedArgStrs = [
				source.get_text_with_indentation(arg)
				for arg in callNode.args]
		else:
			warnings.warn(
				NO_SOURCE_AVAILABLE_WARNING_MESSAGE,
				category=RuntimeWarning, stacklevel=4)
			sanitizedArgStrs = [_absent] * len(args)

		pairs = list(zip(sanitizedArgStrs, args))

		out = self._constructArgumentOutput(prefix, context, pairs)
		return out

	def _constructArgumentOutput(self, prefix, context, pairs):
		"""Construct argument output."""
		def argPrefix(arg):
			return '%s: ' % arg

		pairs = [(arg, self.argToStringFunction(val)) for arg, val in pairs]
		pairStrs = [
			val if (isLiteral(arg) or arg is _absent)
			else (argPrefix(arg) + val)
			for arg, val in pairs]

		allArgsOnOneLine = self._pairDelimiter.join(pairStrs)
		multilineArgs = len(allArgsOnOneLine.splitlines()) > 1

		contextDelimiter = self.contextDelimiter if context else ''
		allPairs = prefix + context + contextDelimiter + allArgsOnOneLine
		firstLineTooLong = len(allPairs.splitlines()[0]) > self.lineWrapWidth

		if multilineArgs or firstLineTooLong:
			if context:
				lines = [prefix + context] + [
					formatPair(len(prefix) * ' ', arg, value)
					for arg, value in pairs
				]
			else:
				argLines = [
					formatPair('', arg, value)
					for arg, value in pairs
				]
				lines = prefixFirstLineIndentRemaining(prefix, '\n'.join(argLines))
		else:
			lines = [prefix + context + contextDelimiter + allArgsOnOneLine]

		return '\n'.join(lines)

	def _formatContext(self, callFrame):
		"""Function for format call frame."""
		filename, lineNumber, parentFunction = self._getContext(callFrame)

		if parentFunction != '<module>':
			parentFunction = '%s()' % parentFunction

		context = f'{filename}:{lineNumber} in {parentFunction}'
		return context

	def _formatTime(self):
		"""Function for format time."""
		now = datetime.now()
		formatted = now.strftime('%H:%M:%S.%f')[:-3]
		return ' at %s' % formatted

	def _getContext(self, callFrame):
		"""Get context of call frame."""
		frameInfo = inspect.getframeinfo(callFrame)
		lineNumber = frameInfo.lineno
		parentFunction = frameInfo.function

		filepath = (realpath if self.contextAbsPath else basename)(frameInfo.filename)
		return filepath, lineNumber, parentFunction

	def enable(self):
		"""Enable pydbg_obj."""
		self.enabled = True

	def disable(self):
		"""Disable pydbg_obj."""
		self.enabled = False

	def configureOutput(self, prefix=_absent, outputFunction=_absent,
						argToStringFunction=_absent, includeContext=_absent,
						contextAbsPath=_absent):
		"""Configure output of pydbg_obj."""
		noParameterProvided = all(
			v is _absent for k,v in locals().items() if k != 'self')
		if noParameterProvided:
			raise TypeError('configureOutput() missing at least one argument')

		if prefix is not _absent:
			self.prefix = prefix

		if outputFunction is not _absent:
			self.outputFunction = outputFunction

		if argToStringFunction is not _absent:
			self.argToStringFunction = argToStringFunction

		if includeContext is not _absent:
			self.includeContext = includeContext
		
		if contextAbsPath is not _absent:
			self.contextAbsPath = contextAbsPath


class Logger:
	"""Logger class: print debug info and save this info to log."""

	def __init__(self, filename: str='python.log', level: str='debug'):
		"""Initiliazation."""
		self.filename = filename
		self.level = level

		if self.level.lower() not in ['debug', 'info', 'warn', 'error', 'critical']:
			raise ValueError(f'Level of logging {self.level} is not supported. Please use debug, info, warn of error')
		else:
			match self.level:
				case 'debug':
					self.level = logging.DEBUG
				case 'info':
					self.level = logging.INFO
				case 'warn':
					self.level = logging.WARNING
				case 'error':
					self.level = logging.ERROR
				case 'critical':
					self.level = logging.CRITICAL
				case _:
					self.level = logging.DEBUG

			logging.getLogger(__name__)
			self.load_config(self.level, self.filename, 'a', "%(name)s/%(module)s at %(lineno)d [%(levelname)s %(asctime)s] %(message)s (function %(funcName)s)")

	def load_config(self, level, filename: str, filemode: str, log_format: str):
		"""Load basic config for logging.

		Arguments:
		---------
		 + level - level of logging (debug, info, warn, error, critical)
		 + filename: str - logfile name
		 + filemode: str - logfile mode (a, w, wb...)
		 + log_format: str - format string for log messages

		"""
		logging.basicConfig(level=level, filename=filename, filemode=filemode, format=log_format)

	def write_to_log(self, message: str) -> bool:
		"""Helper function for writing to a log file.

		Arguments:
		---------
		 + message: str - message for write to log

		Return:
		------
		+ bool - True if successful, False otherwise

		"""
		try:
			with open(self.filename, 'a', encoding='utf8') as file:
				file.write(str(message) + '\n')
		except Exception as e:
			error_message(f'An error occurred while saving logs: {e}')
			return False
		else:
			return True

	def log(self, message: str, msg_type: str, highlight: bool=False):
		"""Log function.

		Arguments:
		---------
		 + message: str - text of message
		 + msg_type: str - type of message
		+ highlight: bool=False - need to highlight text

		"""
		msg_type = msg_type.lower()

		if msg_type == 'info':
			info_message(message, highlight)
			logging.info(message)
		elif msg_type == 'warn':
			warn_message(message, highlight)
			logging.warning(message)
		elif msg_type == 'error':
			error_message(message, highlight)
			logging.error(message)
		elif msg_type == 'debug':
			debug_message(message, highlight)
			logging.debug(message)
		elif msg_type == 'exception':
			run_exception(message, highlight)
			logging.critical(message)
		else:
			other_message(message, msg_type.upper(), highlight)

	def debug_func(self, func):
		"""Decorator for print info about function.

		Arguments:
		---------
		+ func - executed func

		"""
		def wrapper():
			func()
		message = f'debug @ Function {func}() executed at {datetime.now()}'
		self.log(message, 'debug', True)
		return wrapper


def benchmark(func):
	"""Measuring the speed of function execution (decorator).

	Arguments:
	---------
	+ func - executed func

	"""
	start = monotonic()
	def wrapper():
		func()
	end = monotonic()
	total = round(end - start, 5)
	debug_message(f'benchmark {func} @ Execution function {func.__name__} time: {total} sec', True)
	return wrapper


pydbg_obj = PyDBG_Obj()




#
# IceCream - Never use print() to debug again
#
# Ansgar Grunseid
# grunseid.com
# grunseid@gmail.com
#
# License: MIT
#

from pygments.style import Style
from pygments.token import (
	Text, Name, Error, Other, String, Number, Keyword, Generic, Literal,
	Comment, Operator, Whitespace, Punctuation)


class CatppuccinMocha(Style):
	BASE03 = '#1e1e2e'
	BASE02 = '#45475a'
	BASE01 = '#6c7086'
	BASE00 = "#9399b2"
	BASE0 = '#a6adc8'
	BASE1 = '#bac2de'
	BASE2 = '#bac2de'
	BASE3 = '#cdd6f4'
	YELLOW  = '#f9e2af'
	ORANGE  = '#fab387'
	RED     = '#f38ba8'
	MAGENTA = '#cba6f7'
	VIOLET  = '#f5c2e7'
	BLUE    = '#89b4fa'
	CYAN    = '#94e2d5'
	GREEN   = '#a6e3a1'

	styles = {
		Text:                   BASE0,
		Whitespace:             BASE03,
		Error:                  RED,
		Other:                  BASE0,

		Name:                   BASE1,
		Name.Attribute:         BASE0,
		Name.Builtin:           BLUE,
		Name.Builtin.Pseudo:    BLUE,
		Name.Class:             BLUE,
		Name.Constant:          YELLOW,
		Name.Decorator:         MAGENTA,
		Name.Entity:            YELLOW,
		Name.Exception:         RED,
		Name.Function:          BLUE,
		Name.Property:          BLUE,
		Name.Label:             BASE0,
		Name.Namespace:         MAGENTA,
		Name.Other:             BASE0,
		Name.Tag:               MAGENTA,
		Name.Variable:          ORANGE,
		Name.Variable.Class:    BLUE,
		Name.Variable.Global:   BLUE,
		Name.Variable.Instance: BLUE,

		String:                 GREEN,
		String.Backtick:        GREEN,
		String.Char:            CYAN,
		String.Doc:             GREEN,
		String.Double:          GREEN,
		String.Escape:          ORANGE,
		String.Heredoc:         GREEN,
		String.Interpol:        ORANGE,
		String.Other:           GREEN,
		String.Regex:           CYAN,
		String.Single:          GREEN,
		String.Symbol:          GREEN,

		Number:                 MAGENTA,
		Number.Float:           MAGENTA,
		Number.Hex:             VIOLET,
		Number.Integer:         MAGENTA,
		Number.Integer.Long:    VIOLET,
		Number.Oct:             VIOLET,

		Keyword:                CYAN,
		Keyword.Constant:       YELLOW,
		Keyword.Declaration:    CYAN,
		Keyword.Namespace:      ORANGE,
		Keyword.Pseudo:         ORANGE,
		Keyword.Reserved:       VIOLET,
		Keyword.Type:           ORANGE,

		Generic:                BASE0,
		Generic.Deleted:        BASE0,
		Generic.Emph:           BASE0,
		Generic.Error:          BASE0,
		Generic.Heading:        BASE0,
		Generic.Inserted:       BASE0,
		Generic.Output:         BASE0,
		Generic.Prompt:         BASE0,
		Generic.Strong:         BASE0,
		Generic.Subheading:     BASE0,
		Generic.Traceback:      BASE0,

		Literal:                BASE0,
		Literal.Date:           BASE0,

		Comment:                BASE01,
		Comment.Multiline:      BASE01,
		Comment.Preproc:        BASE01,
		Comment.Single:         BASE01,
		Comment.Special:        BASE01,

		Operator:               BASE0,
		Operator.Word:          GREEN,

		Punctuation:            BASE0,
	}


class SolarizedDark(Style):
	BASE03  = '#002b36' # noqa
	BASE02  = '#073642' # noqa
	BASE01  = '#586e75' # noqa
	BASE00  = '#657b83' # noqa
	BASE0   = '#839496' # noqa
	BASE1   = '#93a1a1' # noqa
	BASE2   = '#eee8d5' # noqa
	BASE3   = '#fdf6e3' # noqa
	YELLOW  = '#b58900' # noqa
	ORANGE  = '#cb4b16' # noqa
	RED     = '#dc322f' # noqa
	MAGENTA = '#d33682' # noqa
	VIOLET  = '#6c71c4' # noqa
	BLUE    = '#268bd2' # noqa
	CYAN    = '#2aa198' # noqa
	GREEN   = '#859900' # noqa

	styles = {
		Text:                   BASE0,
		Whitespace:             BASE03,
		Error:                  RED,
		Other:                  BASE0,

		Name:                   BASE1,
		Name.Attribute:         BASE0,
		Name.Builtin:           BLUE,
		Name.Builtin.Pseudo:    BLUE,
		Name.Class:             BLUE,
		Name.Constant:          YELLOW,
		Name.Decorator:         ORANGE,
		Name.Entity:            ORANGE,
		Name.Exception:         ORANGE,
		Name.Function:          BLUE,
		Name.Property:          BLUE,
		Name.Label:             BASE0,
		Name.Namespace:         YELLOW,
		Name.Other:             BASE0,
		Name.Tag:               GREEN,
		Name.Variable:          ORANGE,
		Name.Variable.Class:    BLUE,
		Name.Variable.Global:   BLUE,
		Name.Variable.Instance: BLUE,

		String:                 CYAN,
		String.Backtick:        CYAN,
		String.Char:            CYAN,
		String.Doc:             CYAN,
		String.Double:          CYAN,
		String.Escape:          ORANGE,
		String.Heredoc:         CYAN,
		String.Interpol:        ORANGE,
		String.Other:           CYAN,
		String.Regex:           CYAN,
		String.Single:          CYAN,
		String.Symbol:          CYAN,

		Number:                 CYAN,
		Number.Float:           CYAN,
		Number.Hex:             CYAN,
		Number.Integer:         CYAN,
		Number.Integer.Long:    CYAN,
		Number.Oct:             CYAN,

		Keyword:                GREEN,
		Keyword.Constant:       GREEN,
		Keyword.Declaration:    GREEN,
		Keyword.Namespace:      ORANGE,
		Keyword.Pseudo:         ORANGE,
		Keyword.Reserved:       GREEN,
		Keyword.Type:           GREEN,

		Generic:                BASE0,
		Generic.Deleted:        BASE0,
		Generic.Emph:           BASE0,
		Generic.Error:          BASE0,
		Generic.Heading:        BASE0,
		Generic.Inserted:       BASE0,
		Generic.Output:         BASE0,
		Generic.Prompt:         BASE0,
		Generic.Strong:         BASE0,
		Generic.Subheading:     BASE0,
		Generic.Traceback:      BASE0,

		Literal:                BASE0,
		Literal.Date:           BASE0,

		Comment:                BASE01,
		Comment.Multiline:      BASE01,
		Comment.Preproc:        BASE01,
		Comment.Single:         BASE01,
		Comment.Special:        BASE01,

		Operator:               BASE0,
		Operator.Word:          GREEN,

		Punctuation:            BASE0,
	}


class GruvboxDark(Style):
	BASE03  = '#1d2021' # noqa
	BASE02  = '#282828' # noqa
	BASE01  = '#3c3836' # noqa
	BASE00  = '#4c4846' # noqa
	BASE0   = '#ebdbb2' # noqa
	BASE1   = '#ebdbb2' # noqa
	BASE2   = '#ebbdb2' # noqa
	BASE3   = '#ebdbb2' # noqa
	YELLOW  = '#fabd2f' # noqa
	ORANGE  = '#fabd2f' # noqa
	RED     = '#FB4934' # noqa
	MAGENTA = '#d3869b' # noqa
	VIOLET  = '#d3869b' # noqa
	BLUE    = '#83a598' # noqa
	CYAN    = '#8ec07c' # noqa
	GREEN   = '#b8bb26' # noqa

	styles = {
		Text:                   BASE0,
		Whitespace:             BASE03,
		Error:                  RED,
		Other:                  BASE0,

		Name:                   BASE1,
		Name.Attribute:         BASE0,
		Name.Builtin:           BLUE,
		Name.Builtin.Pseudo:    BLUE,
		Name.Class:             BLUE,
		Name.Constant:          YELLOW,
		Name.Decorator:         ORANGE,
		Name.Entity:            ORANGE,
		Name.Exception:         ORANGE,
		Name.Function:          BLUE,
		Name.Property:          BLUE,
		Name.Label:             BASE0,
		Name.Namespace:         YELLOW,
		Name.Other:             BASE0,
		Name.Tag:               GREEN,
		Name.Variable:          ORANGE,
		Name.Variable.Class:    BLUE,
		Name.Variable.Global:   BLUE,
		Name.Variable.Instance: BLUE,

		String:                 CYAN,
		String.Backtick:        CYAN,
		String.Char:            CYAN,
		String.Doc:             CYAN,
		String.Double:          CYAN,
		String.Escape:          ORANGE,
		String.Heredoc:         CYAN,
		String.Interpol:        ORANGE,
		String.Other:           CYAN,
		String.Regex:           CYAN,
		String.Single:          CYAN,
		String.Symbol:          CYAN,

		Number:                 CYAN,
		Number.Float:           CYAN,
		Number.Hex:             CYAN,
		Number.Integer:         CYAN,
		Number.Integer.Long:    CYAN,
		Number.Oct:             CYAN,

		Keyword:                GREEN,
		Keyword.Constant:       GREEN,
		Keyword.Declaration:    GREEN,
		Keyword.Namespace:      ORANGE,
		Keyword.Pseudo:         ORANGE,
		Keyword.Reserved:       GREEN,
		Keyword.Type:           GREEN,

		Generic:                BASE0,
		Generic.Deleted:        BASE0,
		Generic.Emph:           BASE0,
		Generic.Error:          BASE0,
		Generic.Heading:        BASE0,
		Generic.Inserted:       BASE0,
		Generic.Output:         BASE0,
		Generic.Prompt:         BASE0,
		Generic.Strong:         BASE0,
		Generic.Subheading:     BASE0,
		Generic.Traceback:      BASE0,

		Literal:                BASE0,
		Literal.Date:           BASE0,

		Comment:                BASE01,
		Comment.Multiline:      BASE01,
		Comment.Preproc:        BASE01,
		Comment.Single:         BASE01,
		Comment.Special:        BASE01,

		Operator:               BASE0,
		Operator.Word:          GREEN,

		Punctuation:            BASE0,
	}
